Crate all_is_cubes_base

Crate all_is_cubes_base 

This library is an internal component of all-is-cubes, which defines some core mathematical types and functions. Do not depend on this library; use only all-is-cubes instead.

Modules

math
Do not use this module directly; its contents are re-exported from all-is-cubes. Mathematical utilities and decisions.
raycast
Do not use this module directly; its contents are re-exported from all-is-cubes.
resolution
Do not use this module directly; its contents are re-exported from all-is-cubes.
time
Do not use this module directly; its contents are re-exported from all-is-cubes.
util
Do not use this module directly; its contents are re-exported from all-is-cubes. Tools that we could imagine being in the Rust standard library, but aren’t.

Macros

rgb01
Allows writing a constant Rgb01 color value, provided that its components are float literals.
rgb_const
Allows writing a constant Rgb color value, provided that its components are float literals.
rgba_const
Allows writing a constant Rgba color value, provided that its components are float literals.