#[repr(u8)]pub enum Axis {
X = 0,
Y = 1,
Z = 2,
}Expand description
Enumeration of the axes of three-dimensional space.
Can be used to infallibly index 3-component arrays and vectors.
See also:
Face6specifies an axis and a direction on the axis.
Variants
Implementations
Sourceimpl Axis
impl Axis
Sourcepub fn color(self) -> Rgb01
pub fn color(self) -> Rgb01
Returns a standard color to denote this axis among the three axes. All colors have equal luminance.
- X = red
- Y = green
- Z = blue
Sourcepub fn positive_face(self) -> Face6
pub fn positive_face(self) -> Face6
Returns the Face6 value which corresponds to the positive direction on this axis.
Sourcepub fn negative_face(self) -> Face6
pub fn negative_face(self) -> Face6
Returns the Face6 value which corresponds to the negative direction on this axis.