OctantMap

Struct OctantMap 

pub struct OctantMap<T>(/* private fields */);
Collection of 8 values keyed by Octants.

If the values are bool, use OctantMask instead for a more efficient representation.

Implementations

Source

impl<T> OctantMap<T>

pub fn from_fn(function: impl FnMut(Octant) -> T) -> Self

Constructs an OctantMap by using the provided function to compute a value for each octant.

pub fn repeat(value: T) -> Self
where T: Clone,

Constructs an OctantMap by cloning the provided value.

pub fn to_mask(&self, predicate: impl FnMut(&T) -> bool) -> OctantMask

Returns an OctantMask constructed by applying the given predicate to each octant of the data in self.

pub fn iter(&self) -> impl Iterator<Item = (Octant, &T)> + '_

Returns an iterator over all elements by reference, and their octants.

The order of iteration is not currently guarateed.

pub fn values(&self) -> impl Iterator<Item = &T> + '_

Returns an iterator over all elements by reference.

The order of iteration is not currently guarateed.

pub fn iter_mut(&mut self) -> impl Iterator<Item = (Octant, &mut T)> + '_

Returns an iterator over all elements by mutable reference.

The order of iteration is not currently guarateed.