pub struct PositiveSign<T>(/* private fields */);Expand description
A floating-point number which is not NaN and whose sign bit is positive.
The allowed values consist of positive zero, positive infinity, and every value in between those. This set of values means that this type:
- Implements
Eqstraightforwardly; neither NaN nor signed zeroes can cause problems with using it as a map key for caching, interning, etc. - Is closed under multiplication and addition, so unlike
NotNan, cannot cause a panic from uncautious arithmetic.
The permitted values of a PositiveSign<T> are a subset of those of a NotNan<T>.
(This may not be guaranteed if T implements numeric traits in inconsistent ways,
but may be assumed for f32 and f64.)
The arithmetic behavior of PositiveSign<T> is not identical to T.
Specifically, the value of 0. * fXX::INFINITY is NaN, but the value of
PositiveSign(0.) * PositiveSign(fXX::INFINITY) is PositiveSign(0.) instead.
Our assumption here is that for the applications where PositiveSign is suitable, infinities
are either impossible or arise as “would be finite but it is too large to be represented”,
and do not arise as the reciprocal of zero, and thus we can treat “zero times anything
is zero” as being a more important property than “infinity times anything is infinity”.
Implementations
Sourceimpl<T: FloatCore> PositiveSign<T>
impl<T: FloatCore> PositiveSign<T>
Sourcepub const unsafe fn new_unchecked(value: T) -> Self
pub const unsafe fn new_unchecked(value: T) -> Self
Construct PositiveSign without checking the value.
Safety
value must not be NaN and must have a positive sign bit.
Note that value >= 0. is not a sufficient condition,
because it does not exclude negative zero.
Sourcepub const fn into_inner(self) -> T
pub const fn into_inner(self) -> T
Unwraps the value without modifying it.
Sourcepub fn round(self) -> Self
pub fn round(self) -> Self
Returns the nearest integer to self.
If a value is half-way between two integers, round up from 0.0.
Sourceimpl PositiveSign<f32>
impl PositiveSign<f32>
Sourcepub const fn new_strict(value: f32) -> Self
pub const fn new_strict(value: f32) -> Self
Wraps the given value in PositiveSign.
- If
valueis positive (including positive infinity), returns wrappedvalue. - If
valueis zero of either sign, returns wrapped positive zero. This is lossy, but corresponds to the IEEE 754 idea that -0.0 == +0.0. - If
valueis negative non-zero or NaN, panics.
Sourcepub const fn new_clamped(value: f32) -> Self
pub const fn new_clamped(value: f32) -> Self
Wraps the given value in PositiveSign.
- If the value is NaN, panics.
- If the value has a negative sign, it is replaced with positive zero.
Sourcepub fn saturating_sub(self, other: Self) -> Self
pub fn saturating_sub(self, other: Self) -> Self
Subtract other from self; if the result would be negative, it is zero instead.
Sourceimpl PositiveSign<f64>
impl PositiveSign<f64>
Sourcepub const fn new_strict(value: f64) -> Self
pub const fn new_strict(value: f64) -> Self
Wraps the given value in PositiveSign.
- If
valueis positive (including positive infinity), returns wrappedvalue. - If
valueis zero of either sign, returns wrapped positive zero. This is lossy, but corresponds to the IEEE 754 idea that -0.0 == +0.0. - If
valueis negative non-zero or NaN, panics.
Sourcepub const fn new_clamped(value: f64) -> Self
pub const fn new_clamped(value: f64) -> Self
Wraps the given value in PositiveSign.
- If the value is NaN, panics.
- If the value has a negative sign, it is replaced with positive zero.
Sourcepub fn saturating_sub(self, other: Self) -> Self
pub fn saturating_sub(self, other: Self) -> Self
Subtract other from self; if the result would be negative, it is zero instead.